// Copyright (C) 1997, 1998 by DI Wolfgang Platzer, IAIK
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package iaik.jigsaw.https;

import java.util.Hashtable;
import java.util.Vector;
import java.security.Principal;
import java.security.InvalidKeyException;
import java.security.cert.*;

import iaik.security.ssl.SSLCertificate;

import iaik.x509.SimpleChainVerifier;

// the casts from java.security.cert.X509Certificate to iaik.x509.X509Certificate
// are only necessary because of the API declaration in IAIK JCE 2.04. This 
// should change in the next version.

/**
 * This class implements a ServerTrustDecider for Jigsaw.
 */
public class JigsawServerTrustDecider implements iaik.security.ssl.ServerTrustDecider {
  
  private final static boolean DEBUG = true;

  private SimpleChainVerifier chainVerifier;
  private boolean requireCertificate = false;
  private boolean requireTrustedRoot = false;
  
  public JigsawServerTrustDecider() {
    chainVerifier = new SimpleChainVerifier();
  }
  
  public boolean isTrustedPeer(SSLCertificate certificate) {
    if( DEBUG ) System.out.println("JigawServerTrustDecider: isTrustedPeer() called");
    if( certificate == null ) {
      if( requireCertificate ) {
        if( DEBUG ) System.out.println("No client certificate, refusing connection");
        return false;
      } else {
        if( DEBUG ) System.out.println("No client certificate, may connect anyway.");
        return true;
      }
    }
    iaik.x509.X509Certificate[] serverChain = castCertificates(certificate.getCertificateChain());
    try {
      boolean result = chainVerifier.verifyChain(serverChain);
      if( result ) {
        if( DEBUG ) System.out.println("Client certificate has chain a trusted root, good.");
        return true;
      }
      if( requireTrustedRoot ) {
        if( DEBUG ) System.out.println("No trusted root certificate, connection NOT allowed!");
        return false;
      } else {
        if( DEBUG ) System.out.println("No trusted root certificate, but connect allowed anyway.");
        return true;
      }
    } catch( CertificateException e ) {
      if( DEBUG ) {
        System.out.println("Exception during chain verification, returning false");
        System.out.println(e);
      }
      return false;
    }
  }

  private static iaik.x509.X509Certificate[] castCertificates(Object[] objs) {
    iaik.x509.X509Certificate[] certs = new iaik.x509.X509Certificate[objs.length];
    for( int i=0; i<objs.length; i++ ) {
      certs[i] = (iaik.x509.X509Certificate)objs[i];
    }
    return certs;
  }

  
  /**
   * Must the client present a certificate.
   *
   * If this value is set to true the client has to send a certifcate.
   *
   * @param required true, if a client certificate is required, false otherwise
   */
  public void setRequireCertificate(boolean required) {
    if (DEBUG) {
      System.out.println("setRequireCertificate to "+required);
    }
    requireCertificate = required;
  }
  
  /**
   * Must the client certificate be signed from an trusted signer.
   *
   * If this value is set to true the client has to present a certificate, which
   * was signed from an trusted signer.
   *
   * @param required true, if a trusted signer must be present, false otherwise
   */
  public void setRequireTrustedRoot(boolean required) {
    if (DEBUG) {
      System.out.println("setRequireTrustedRoot to "+required);
    }
    requireTrustedRoot = required;
  }

  /**
   * Set the list with the certificates of the trusted signers.
   * Previous trusted signers are deleted.
   *
   * @param certificateList the list of certificates
   */
  public void setTrustedSigners(X509Certificate[] certificateList) {
    chainVerifier = new SimpleChainVerifier();
    for (int i=0; i<certificateList.length; i++) {
      addTrustedSigner(certificateList[i]);
    }
  }
  
  /**
   * Add one trusted signer.
   *
   * @param certificate the certificate to be added as trusted
   */
  public void addTrustedSigner(X509Certificate certificate) {
    Principal dn = certificate.getSubjectDN();
    if (DEBUG) {
      System.out.println("Adding dn: " + dn);
    }
    chainVerifier.addTrustedCertificate((iaik.x509.X509Certificate)certificate);
  }
}
